<?php

namespace modules\catalogs\services;

use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;
use modules\catalogs\Models\ProductLocation;
use modules\catalogs\Models\Products;
use modules\catalogs\Models\ProductStock;
use modules\location\Models\Location;
use modules\stocks\Models\StockControl;
use modules\stocks\Models\StockStory;

class StockService
{
    /**
     * Met à jour le stock d'un produit.
     *
     * @param array $data
     * @param string $type 'entry' ou 'exit'
     * @return Product
     * @throws ValidationException
     */
    public function update(array $data, string $type): array
    {
        #dd($data);
        //Récupération du produit
        $product = ProductService::products(['id' => $data['product_id'], 'location' => $data['location_id']]);
        $product = $product[0]->toArray();        
        
        if(!$product){
            throw ValidationException::withMessages([
                'product_id' => 'Le produit spécifié est introuvable.',
            ]);
        }

        //Validation des données d'entrée
        if ($data['quantity'] <= 0 && $type != 'back') {
            throw ValidationException::withMessages([
                'quantity' => 'La quantité doit être supérieure à zéro.',
            ]);
        }
        
        if ($type == 'entry') {
            $this->handleEntry($product, $data['location_id'], $data['quantity'], $data['price']);
        } elseif ($type == 'exit' || $type == 'back') {            
            $this->handleExit($product, $data['location_id'], $data['quantity'], $type);
        } elseif ($type == 'inventory') {
            $this->handleInventory($product, $data['location_id'], $data['quantity']);
        } else {
            throw ValidationException::withMessages([
                'type' => "Le type d'opération doit être 'entry' ou 'exit'.",
            ]);
        }

        return $product;
    }

    public function logStockStory(array $data): void
    {
        $location = Location::find($data['location_id']);
        $user = Auth::user();

        StockStory::create([
            'product_id' => $data['product_id'],
            'product_name' => $data['product_name'],
            'location_id' => $data['location_id'],
            'location_name' => $location->title ?? 'Unknown',
            'title' => $data['title'],
            'stock_actual' => $data['stock_actual'],
            'stock_mvt' => $data['quantity'],
            'stock_final' => $data['stock_final'],
            'user_id' => $user->id,
            'username' => $user->name,
            'token' => $data['token'],
            'created_at' => date_now(),
            'updated_at' => date_now(),
        ]);
    }

    public function createUpdateControl(array $data): void {
        $where = $data['where'] ?? [];
        $location = Location::find($data['location_id']);
        $user = Auth::user();

        $control = StockControl::where($where)
                    ->whereDate('created_at', date_now()->toDateString());
        #dd($control);

        if($data['type'] == 'in'){                
            $query['stock_in'] = DB::raw("stock_in + " . (double) $data['quantity']);
            $query['stock_final'] = DB::raw("stock_final - " . (double) $data['quantity']);
        }

        if($data['type'] == 'out'){
            $query['stock_out'] = DB::raw("stock_out + " . (double) $data['quantity']);                    
            $query['stock_final'] = DB::raw("stock_final - " . (double) $data['quantity']);
        }

        if($data['type'] == 'hs'){            
            $query['stock_hs'] = DB::raw("stock_hs + " . (double)$data['quantity']);
            $query['stock_final'] = $data['stock_final'];
        }

        if($data['type'] == 'inventory'){
            $query['stock_inventory'] = DB::raw("stock_inventory + " . (double)$data['quantity']);
            $query['stock_final'] = (double)$data['quantity']; // Le stock final devient le stock d'inventaire
        }

        if (!is_null($control->first())) {
            $query['updated_at'] = date_now();
            $control->update($query);
        } else {
            $query['product_id'] = $data['product_id'];
            $query['product_name'] = $data['product_name'];
            $query['location_id'] = $data['location_id'];
            $query['location_name'] = $location->title ?? 'Unknown';
            $query['stock_actual'] = $data['stock_actual'];                            
            $query['user_id'] = $user->id;
            $query['username'] = $user->name;
            $query['created_at'] = date_now();
            $query['updated_at'] = date_now();            
           
            StockControl::create($query);
        }
    }

     /**
     * Gère les mises à jour d'inventaire.
     *
     * @param array $product
     * @param int $location_id
     * @param float $quantity
     * @return void
     */
    private function handleInventory(array $product, int $location_id, float $quantity): void
    {
        $condition = [
            'product_id' => $product['id'],
            'location_id' => $location_id,            
        ];

        // Mise à jour directe du stock avec la quantité fournie
        ProductStock::where($condition)->update([
            'stock' => $quantity,
            'updated_at' => date_now(),
        ]);
    }

    /**
     * Gère les entrées de stock avec calcul du CMUP.
     *
     * @param array $product
     * @param int $quantity
     * @param float $unitPrice
     * @return void
     */
    private function handleEntry(array $product, int $location_id, float $quantity, float $unitPrice): void
    {
        $currentStock = $product['stock'][0]['stock'] ?? 0;
        $currentCmup = $product->cmup ?? 0;

        // Calcul du nouveau stock et du CMUP
        $newStock = $currentStock + $quantity;        
        $newCmup = (($currentStock * $currentCmup) + ($quantity * $unitPrice)) / $newStock;        

        $condition = [
            'product_id' => $product['id'],
            'location_id' => $location_id,            
        ];

        // Mise à jour stock du produit
        ProductStock::where($condition)->update([
            'stock' => DB::raw("stock + {$quantity}"),
            'updated_at' => date_now(),

        ]);

        ProductLocation::where($condition)->update([
            'cmup' => $newCmup,
        ]);        
    }

    /**
     * Gère les sorties de stock.
     *
     * @param Products $product
     * @param int $quantity
     * @return void
     */
    private function handleExit(array $product, int $location_id, float $quantity, string $type = 'exit'): void
    {
        #dd('ato');
        $currentStock = $product['stock'][0]['stock'] ?? 0;
        #dd($quantity, $currentStock, $type);

        if ($type == 'exit' and $quantity > $currentStock) {
            throw ValidationException::withMessages([
                'quantity' => 'Quantité insuffisante en stock.',
            ]);
        }

        #dd('"');

        // Mise à jour du stock
        $newStock = $currentStock - $quantity;
        $condition = [
            'product_id' => $product['id'],
            'location_id' => $location_id,            
        ];     
        #dd($currentStock, $quantity);

        // Mise à jour stock du produit
        ProductStock::where($condition)->update([
            'stock' => $newStock,
            'updated_at' => date_now(),
        ]);
    }
}